/*
 *      Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 *      Licensed under the Flora License, Version 1.1 (the "License");
 *      you may not use this file except in compliance with the License.
 *      You may obtain a copy of the License at
 *
 *              http://floralicense.org/license/
 *
 *      Unless required by applicable law or agreed to in writing, software
 *      distributed under the License is distributed on an "AS IS" BASIS,
 *      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *      See the License for the specific language governing permissions and
 *      limitations under the License.
 */

/*global moonphase */

/**
 * App.js controls main page display, including changes in date information.
 */
(function() {

    var timerUpdateDate = 0,

        /**
         * First rotate angle for new moon.
         * @type {number}
         */
        NEW_MOON_ANGLE = 60;

    /**
     * Rotates the angle corresponding to the id.
     * @private
     * @param {number} angle
     * @param {string} id
     */
    function rotateElement(angle, id) {
        document.querySelector("#" + id).style.transform = "rotate(" + angle + "deg)";
    }

    /**
     * Updates moon phase using date information.
     * @private
     */
    function updateMoon() {
        var datetime = tizen.time.getCurrentDateTime(),
            dateInformation = {
                "year": "",
                "month": "",
                "date": ""
            },
            rotateAngle; // rotate angle for moon phase.

        // getFullYear() function returns current year.
        dateInformation.year = datetime.getFullYear();

        // getMonth() function returns 0 to 11 as current month. (0 is January)
        dateInformation.month = datetime.getMonth() + 1;

        // getDate() function returns current date. (1 - 31)
        dateInformation.date = datetime.getDate();

        if(dateInformation.month < 10) {
            dateInformation.month = "0" + dateInformation.month;
        }

        if(dateInformation.date < 10) {
            dateInformation.date = "0" + dateInformation.date;
        }

        // parameter of moonphase.inputDate() is date information,
        // then it returns rotate angle based on new moon.
        rotateAngle = moonphase.inputDate(dateInformation);

        rotateElement(NEW_MOON_ANGLE + rotateAngle, "moonphase-moon");
    }

    /**
     * Updates the date and sets refresh callback on the next day.
     * @private
     * @param {number} prevDay - date of the previous day
     */
    function updateDate(prevDay) {
        var datetime = tizen.time.getCurrentDateTime(),
            nextInterval,
            dateNumStr = document.querySelector('#date-num-string');

        // Check the update condition.
        // if prevDate is '0', it will always update the date.
        if (prevDay !== null) {
            if (prevDay === datetime.getDay()) {
                /**
                 * If the date was not changed (meaning that something went wrong),
                 * call updateDate again after a second.
                 */
                nextInterval = 1000;
            } else {
                /**
                 * If the day was changed,
                 * call updateDate at the beginning of the next day.
                 */
                // Calculate how much time is left until the next day.
                nextInterval =
                    (23 - datetime.getHours()) * 60 * 60 * 1000 +
                    (59 - datetime.getMinutes()) * 60 * 1000 +
                    (59 - datetime.getSeconds()) * 1000 +
                    (1000 - datetime.getMilliseconds()) +
                    1;
            }
        }

        // update date string.
        dateNumStr.innerHTML = datetime.getDate();

        // update month stroke.
        rotateElement( 20 * (datetime.getMonth() ), "body-calendar-stroke");

        // update day hand and shadow.
        rotateElement( 37.5 * ( (datetime.getDay() - 3 ) ), "body-calendar-hand");
        rotateElement( 37.5 * ( (datetime.getDay() - 3 ) ), "body-calendar-hand_shadow");

        // If an updateDate timer already exists, clear the previous timer.
        if (timerUpdateDate) {
            clearTimeout(timerUpdateDate);
        }

        // Set next timeout for date update.
        timerUpdateDate = setTimeout(function() {
            updateDate(datetime.getDay());
            updateMoon();
        }, nextInterval);
    }

    /**
     * Updates the current time.
     * @private
     */
    function updateTime() {
        var datetime = tizen.time.getCurrentDateTime(),
            hour = datetime.getHours(),
            minute = datetime.getMinutes(),
            second = datetime.getSeconds();

        // update the hour/minute/second hands and shadows.
        rotateElement((hour + (minute / 60) + (second / 3600)) * 30, "body-hr-hand");
        rotateElement((hour + (minute / 60) + (second / 3600)) * 30, "body-hr-hand-shadow");
        rotateElement((minute + second / 60) * 6, "body-min-hand");
        rotateElement((minute + second / 60) * 6, "body-min-hand-shadow");
        rotateElement(second * 6, "body-sec-hand");
        rotateElement(second * 6, "body-sec-hand-shadow");
    }

    /**
     * Updates watch screen. (time, date and moon phase)
     */
    function updateWatch() {
        updateTime();
        updateDate(0);
        updateMoon();
    }

    /**
     * Binds events.
     */
    function bindEvents() {
        // add eventListener to update the screen immediately when the device wakes up.
        document.addEventListener("visibilitychange", function() {
            if (!document.hidden) {
                updateWatch();
            }
        });

        // add event listeners to update watch screen when the time zone is changed.
        tizen.time.setTimezoneChangeListener(function() {
            updateWatch();
        });
    }

    /**
     * Initializes date, moon phase and time.
     * @private
     */
    function init() {
        updateDate(0);
        updateMoon();
        setInterval(function() {
            updateTime();
        }, 1000);

        bindEvents();
    }

    window.onload = init();

}());